#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "glext.h"

#include "intro.h"
#include "mesh.h"
#include "mzk.h"

//---------------------------------------------------------------------

#define ANCHO   32

#define fzn  0.005f
#define fzf  1000.0f
#define ftf  0.8f

float projectionmatrix[16] = {
    1.0f/ftf, 0.0f,     0.0f,                     0.0f,
    0.0f,     1.0f/ftf, 0.0f,                     0.0f,
    0.0f,     0.0f,     -(fzf+fzn)/(fzf-fzn),    -1.0f,
    0.0f,     0.0f,     -2.0f*fzf*fzn/(fzf-fzn),  0.0f };
VEC3D camtar = { 0.0f, -0.1f, 0.0f };


const float colors[3*4] = { 0.25f, 0.25f, 0.40f,   
                                   1.60f, 0.50f, 0.30f,   
                                   0.80f, 1.50f, 0.90f,    
                                   1.50f, 0.90f, 0.50f };

const float rads[3]   = { 0.116f*0.116f, 0.116f*0.116f, 0.14f*0.14f };

const VEC3D center[3] = { { 0.43f, -0.08f, -0.04f}, {-0.25f, -0.35f, 0.10f}, { -0.25f, -0.13f,-0.24f} };

//---------------------------------------------------------
#define NUMFUNCIONES    2

unsigned char buffer[ANCHO*ANCHO*4];
float buf_colors[4+1][MAXVERTS*4];
float buf_tmp[MAXVERTS*4];
void *myglfunc[NUMFUNCIONES];
QMESH _qm;
int globalviewport[4];


#define oglBindBufferARB    ((PFNGLBINDBUFFERARBPROC)myglfunc[0])
#define oglBufferDataARB    ((PFNGLBUFFERDATAARBPROC)myglfunc[1])

//---------------------------------------------------------

void renderFromHere( const VEC3D *tar, const VEC3D *pos, int c )
{
    projectionmatrix[5] =1.0f/ftf;
    if( c<4 ) projectionmatrix[5]*= 0.75f*float(XRES)/float(YRES);

    glLoadMatrixf( projectionmatrix );
    gluLookAt( pos->x, pos->y, pos->z, tar->x, tar->y, tar->z, 0.0f, 1.0f, 0.0f );

    oglBindBufferARB( GL_ARRAY_BUFFER_ARB, 2+c );
    glColorPointer( 4, GL_FLOAT, 0, 0 );

    glDrawElements( GL_QUADS, 4*MAXFACES, GL_UNSIGNED_INT, 0 );
}

__forceinline void total( float *fcol )
{
    for( unsigned int i=0; i<(4*ANCHO*ANCHO); i++ )
        fcol[i&3] += (float)buffer[i] * (1.4f/(float)(255*ANCHO*ANCHO));
}

void VecSInc( const VEC3D *v1, const float s, VEC3D *dst );

void intro_init( void )
{
    //--- load geometry -----------------------------------

    loadMesh( &_qm);

    //--- colorze -----------------------------------

    for( int i=0; i<MAXVERTS; i++ )
    for( int j=0; j<3; j++ )
    {
        float dis = -rads[j];
        for( int k=0; k<3; k++ )
        {
            const float dx = ((float*)(_qm.verts+i))[k] - ((float*)(center+j))[k];
            dis += dx*dx;
        }

        float f;
        if( dis<0.0f )
            f = 1.0f;
        else
            f = 0.0f;
        buf_colors[4][4*i+1+j] = f;
    }

    //--- upload to vram -----------------------------------

    myglfunc[0] = (void *)wglGetProcAddress( "glBindBufferARB" );
    myglfunc[1] = (void *)wglGetProcAddress( "glBufferDataARB" );

    oglBindBufferARB( GL_ARRAY_BUFFER_ARB, 6 );
    oglBufferDataARB( GL_ARRAY_BUFFER_ARB, MAXVERTS*4*sizeof(float), buf_colors[4], GL_STATIC_DRAW_ARB );
    oglBindBufferARB( GL_ELEMENT_ARRAY_BUFFER_ARB, 7 );
    oglBufferDataARB( GL_ELEMENT_ARRAY_BUFFER_ARB, MAXFACES*sizeof(QUAD), _qm.quads, GL_STATIC_DRAW_ARB );
    oglBindBufferARB( GL_ARRAY_BUFFER_ARB, 1 );
    oglBufferDataARB( GL_ARRAY_BUFFER_ARB, MAXVERTS*sizeof(VEC3D), _qm.verts, GL_STATIC_DRAW_ARB );

    //--- set ogl states ------------------------------

    glGetIntegerv( GL_VIEWPORT, globalviewport );
    glEnable( GL_DEPTH_TEST );
    glEnable( GL_CULL_FACE );
    glDepthFunc( GL_LEQUAL );
    glLineWidth( 4.0f ); 
    glEnableClientState( GL_VERTEX_ARRAY );
    glEnableClientState( GL_COLOR_ARRAY );
//    oglBindBufferARB( GL_ARRAY_BUFFER_ARB, 1 );
    glVertexPointer( 3, GL_FLOAT, 0, 0);//_qm.verts );

    //--- calc AO -----------------------------------

    glViewport( 0, 0, ANCHO, ANCHO );
    glClearColor( 0.75f, 0.0f, 0.0f, 0.0f );

    for( int i=0; i<MAXVERTS; i++ )
    {
        float f = 4.0f + 8.0f*_qm.verts[i].y;
        if( f>1.0f ) f=1.0f;                

        VecSInc( &_qm.verts[i], 1.0f, &_qm.norms[i] );

        glClear( GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT );
        renderFromHere( _qm.norms+i, _qm.verts+i, 4 ); 
        glReadPixels( 0, 0, ANCHO, ANCHO, GL_RGBA, GL_UNSIGNED_BYTE, buffer );
        total( buf_tmp+4*i );

        for( int j=0; j<4; j++ )
        {
            const int k = 4*i+j;
            const float factor = (buf_colors[4][k] + buf_tmp[k])*f;

            for( int h=0; h<3; h++ ) buf_colors[j][4*i+h] = factor*colors[3*j+h];
        }
    }

    for( int i=0; i<4; i++ )
    {
        oglBindBufferARB( GL_ARRAY_BUFFER_ARB, 2+i );
        oglBufferDataARB( GL_ARRAY_BUFFER_ARB, MAXVERTS*4*sizeof(float), buf_colors[i], GL_STATIC_DRAW_ARB );
    }

    //--- prepare state for rendering ---

    glViewport( 0, 0, globalviewport[2], globalviewport[3] );
    glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
}


//----------------------------------------
extern unsigned char notas00[280];
extern unsigned char deltas_b0_00[280];


#define MKB(a,b,c,d,e,f,g) ((g<<6)|(f<<5)|(e<<4)|(d<<3)|(c<<2)|(b<<1)|(a<<0))

int            tiempos2[] = { 0,                      20462,                     21000,                     22431,                    24800,                   25600,                    26200,                      28231,                    29000,                    31000,                    51300,                      61500,                     72000,                       101000,                      105000 };
unsigned char  ll[]       = {      MKB(0,0,0,0,0,1,1),        MKB(1,1,1,1,0,0,1),        MKB(0,0,0,0,0,0,1),       MKB(1,1,1,1,0,0,1),      MKB(0,0,0,0,0,0,0),       MKB(1,1,1,1,0,0,1),        MKB(0,0,0,0,0,0,0),       MKB(1,1,1,1,0,1,1),       MKB(0,0,0,0,0,0,0),       MKB(1,0,0,0,0,0,0),         MKB(1,0,0,0,0,1,0),         MKB(1,0,0,0,1,0,0),         MKB(1,0,0,0,1,1,0),          MKB(0,0,0,0,0,0,0) };

void intro_do( long time )
{
    VEC3D pos;

    int encendido;

    // synchro
    float t = 0.001f*(float)time;
    for( int i=0; i<((sizeof(tiempos2)/4)-1); i++ )
    {
        if( time>=tiempos2[i] && time<tiempos2[i+1] )
        {
            encendido = ll[i];
            if( encendido & (1<<6) ) t = (float)(encendido+i*430);
            break;
        }
    }
    
    // camara variform
    if( encendido & (1<<4) )
    {
        t += 20.00f*(float)(time/(4*160));
        t += 0.002f*(float)(time%(4*160));
    }

    // camara
    pos.x =           0.75f*cosf( .41f*t );
    pos.y = -0.085f + 0.40f*sinf( .12f*t );
    pos.z =           0.75f*sinf( .31f*t );

    //--------------
    // bombillas 
    if( encendido & (1<<5) )
    {
        int tt = 0;
        for( int i=0; i<280; i++ )
        {
            tt += deltas_b0_00[i] * 160;    // 7056*1000/MZK_RATE;

            if( time>tt && time<(tt+280) )
            {
                int id;
                     if( notas00[i]==0x31 ) id = 1;
                else if( notas00[i]==0x38 ) id = 2;
                else                        id = 3;
                encendido |= (1<<id);
            }
        }
    }
    //-------------


    //glViewport( 0, 0, XRES, YRES );
//    glViewport( 0, 0, globalviewport[2], globalviewport[3] );
//    glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );

//    glViewport( 0, 0, globalviewport[2], globalviewport[3] );
//    glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
    glClear( GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT );

    glBlendFunc( GL_ONE, GL_ONE );
    glDisable( GL_BLEND );

    for( int i=0; i<5; i++ )
    {
        if( encendido & (1<<i)  )
        {
            glPolygonMode( GL_FRONT, i==4?GL_LINE:GL_FILL );
            //glPolygonMode( GL_FRONT, GL_LINE + (i!=4) );
            //glPolygonMode( GL_FRONT, GL_FILL - (i>>2) );

            renderFromHere( &camtar, &pos, i&3 );
            glEnable( GL_BLEND );
        }
    }
}

