//--------------------------------------------------------------------------//
// iq . 2003 . code for the Paradise 64 kb intro by RGBA                    //
//--------------------------------------------------------------------------//
#define ANTI        1

#define ERRORMSG            // activa printado mensajes de error
#define LOADING             // activa pintado "wait while loading..."
#define AUTOCENTER          // activa centrado de la ventana

#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>
#include <mmsystem.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <GL/gl.h>
#include "../../intro.h"
#include "../../mzk.h"
#include "raster.h"

//----------------------------------------------------------------------------

typedef struct
{
    //---------------
    HINSTANCE   hInstance;
    HDC         hDC;
    HGLRC       hRC;
    HWND        hWnd;
    //---------------
    int         full;
    //---------------
    char        wndclass[14];	// window class and title :)
    //---------------
}WININFO;


static PIXELFORMATDESCRIPTOR pfd =
    {
    sizeof(PIXELFORMATDESCRIPTOR),
    1,
    PFD_DRAW_TO_WINDOW|PFD_SUPPORT_OPENGL|PFD_DOUBLEBUFFER,
    PFD_TYPE_RGBA,
    32,
    0, 0, 0, 0, 0, 0, 8, 0,
    0, 0, 0, 0, 0,  // accum
    32,             // zbuffer
    0,              // stencil!
    0,              // aux
    PFD_MAIN_PLANE,
    0, 0, 0, 0
    };

static WININFO wininfo = {  0,0,0,0,
                            0,
							{'r','g','b','a','_','p','a','r','a','d','i','s','e',0}
                            };
#ifdef LOADING
static char fnt_wait[6]    = "arial";
static char msg_wait[22]   = "wait while loading...";
#endif
#ifdef ERRORMSG
static char msg_error[207] = "intro_init()!\n\n"\
                             "  no memory?\n"\
                             "  no music?\n"\
                             "  no GL_ARB_multitexture?\n"\
                             "  no GL_ARB_vertex_program?\n"\
                             "  no GL_ARB_fragment_program?\n"\
							 "  no GL_EXT_texture_lod_bias?\n"\
                             "  no GL_EXT_texture3D?\n"\
                             "  no GL_SGIS_generate_mipmap?";

static char tlt_error[6]   = "error";
#endif


static int wavHeader[11] = {
    0x46464952, 
    MZK_NUMSAMPLES*2+36, 
    0x45564157, 
    0x20746D66, 
    16, 
    WAVE_FORMAT_PCM|(MZK_NUMCHANNELS<<16), 
    MZK_RATE, 
    MZK_RATE*MZK_NUMCHANNELS*sizeof(short), 
    (MZK_NUMCHANNELS*sizeof(short))|((8*sizeof(short))<<16),
    0x61746164, 
    MZK_NUMSAMPLES*sizeof(short)
    };

//----------------------------------------------------------------------------

static LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	// salvapantallas
	if( uMsg==WM_SYSCOMMAND && (wParam==SC_SCREENSAVE || wParam==SC_MONITORPOWER) )
		return( 0 );

	// boton x o pulsacion de escape
	if( uMsg==WM_CLOSE || (uMsg==WM_KEYDOWN && wParam==VK_ESCAPE) )
	{
		PostQuitMessage(0);
        return( 0 );
	}

    return( DefWindowProc(hWnd,uMsg,wParam,lParam) );
}

static void window_end( WININFO *info )
{
    if( info->hRC )
    {
        wglMakeCurrent( 0, 0 );
        wglDeleteContext( info->hRC );
    }

    if( info->hDC  ) ReleaseDC( info->hWnd, info->hDC );
    if( info->hWnd ) DestroyWindow( info->hWnd );

    UnregisterClass( info->wndclass, info->hInstance );

    if( info->full )
    {
        ChangeDisplaySettings( 0, 0 );
		while( ShowCursor( 1 )<0 ); // show cursor
    }
}

static int window_init( WININFO *info )
{
	unsigned int	PixelFormat;
    DWORD			dwExStyle, dwStyle;
    DEVMODE			dmScreenSettings;
    WNDCLASS		wc;
    RECT			rec;

    memset( &wc, 0, sizeof(WNDCLASS) );
    wc.style         = CS_OWNDC|CS_HREDRAW|CS_VREDRAW;
    wc.lpfnWndProc   = WndProc;
    wc.hInstance     = info->hInstance;
    wc.lpszClassName = info->wndclass;
    wc.hbrBackground=(HBRUSH)CreateSolidBrush(0x00785838);
	
    if( !RegisterClass(&wc) )
        return( 0 );

    if( info->full )
    {
        memset( &dmScreenSettings,0,sizeof(DEVMODE) );
        dmScreenSettings.dmSize       = sizeof(DEVMODE);
        dmScreenSettings.dmFields     = DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;
        dmScreenSettings.dmBitsPerPel = 32;
        dmScreenSettings.dmPelsWidth  = XRES;
        dmScreenSettings.dmPelsHeight = YRES;

        if( ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
            return( 0 );

        dwExStyle = WS_EX_APPWINDOW;
        dwStyle   = WS_VISIBLE | WS_POPUP;// | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

		while( ShowCursor( 0 )>=0 );	// hide cursor
    }
    else
    {
        dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;
        dwStyle   = WS_VISIBLE | WS_CAPTION | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_SYSMENU;
    }

    rec.left   = 0;
    rec.top    = 0;
    rec.right  = XRES;
    rec.bottom = YRES;

    AdjustWindowRect( &rec, dwStyle, 0 );

    #ifdef AUTOCENTER
        info->hWnd = CreateWindowEx( dwExStyle, wc.lpszClassName, wc.lpszClassName, dwStyle,
		//info->hWnd = CreateWindowEx( dwExStyle, "EDIT", wc.lpszClassName, dwStyle,
                                   (GetSystemMetrics(SM_CXSCREEN)-rec.right+rec.left)>>1,
                                   (GetSystemMetrics(SM_CYSCREEN)-rec.bottom+rec.top)>>1,
                                   rec.right-rec.left, rec.bottom-rec.top, 0, 0, info->hInstance, 0 );
    #else
        info->hWnd = CreateWindowEx( dwExStyle, wc.lpszClassName, wc.lpszClassName, dwStyle,
                                   0, 0, rec.right-rec.left, rec.bottom-rec.top,
                                   0, 0, info->hInstance, 0 );
    #endif

    if( !info->hWnd )
        return( 0 );

    if( !(info->hDC=GetDC(info->hWnd)) )
        return( 0 );

    if( !(PixelFormat=ChoosePixelFormat(info->hDC,&pfd)) )
        return( 0 );

    if( !SetPixelFormat(info->hDC,PixelFormat,&pfd) )
        return( 0 );

    if( !(info->hRC=wglCreateContext(info->hDC)) )
        return( 0 );

    if( !wglMakeCurrent(info->hDC,info->hRC) )
        return( 0 );
    
    //SetForegroundWindow( info->hWnd );    // slightly higher priority
    //SetFocus( info->hWnd );               // sets keyboard focus to the window
    
    return( 1 );
}

//----------------------------------------------------------------------------

static unsigned char *rbuf=0;
int externaltimer;
static int doframe( float *buffer, int xres, int yres, float t )
{
    unsigned char   *ptr;
    int             i;
    MSG				msg;
	int				done = 0;

    //---------------------------------

//    glViewport( 0, 0, xres, yres );
    
	externaltimer = (int)(t*1000.0f);
	intro_do( externaltimer );

    if( t>(MZK_DURATION*1000) )
        done = 1;


    glReadPixels( 0, 0, ANTI*xres, ANTI*yres, GL_RGBA, GL_UNSIGNED_BYTE, rbuf );

    ptr = rbuf;
    for( i=0; i<(ANTI*ANTI*xres*yres); i++ )
    {
        *buffer++ = ptr[0]*(1.0f/255.0f);
        *buffer++ = ptr[1]*(1.0f/255.0f);
        *buffer++ = ptr[2]*(1.0f/255.0f);
        ptr += 4;
    }

    //---------------------------------

    while( PeekMessage(&msg, wininfo.hWnd, 0, 0, PM_REMOVE) )
    {
		if( msg.message==WM_QUIT ) done=1;
        //TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    SwapBuffers( wininfo.hDC );

	return( done );
}

void OUT_Free( void )
{
    free( rbuf );
}


int OUT_Init( int xres, int yres )
{
    rbuf = (unsigned char*)malloc( ANTI*ANTI*xres*yres*4 );
    if( !rbuf )
        return( 0 );

    if( !RASTER_Config( xres, yres, ANTI, 1, 60.0f, 60*MZK_DURATION ) )
        return( 0 );

    return( 1 );
}

//----------------------------------------------------------------------------

short myMuzik[MZK_NUMSAMPLES+22];

//----------------------------------------------------------------------------

int WINAPI WinMain( HINSTANCE instance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow )
{
    int         done=0;
    WININFO     *info = &wininfo;

    info->hInstance = GetModuleHandle( 0 );


    if( !window_init(info) )
    {
        window_end( info );
        #ifdef ERRORMSG
        MessageBox( 0, "window_init()!",tlt_error,MB_OK|MB_ICONEXCLAMATION );
        #endif
        return( 0 );
    }

    intro_init();

    mzk_init( myMuzik+22 );

    memcpy( myMuzik, wavHeader, 44 );

    FILE *fp = fopen( "out/mzk.wav", "wb" );
    if( !fp )
        return( 0 );
    fwrite( &myMuzik, 2, MZK_NUMSAMPLES+22, fp );
    fclose( fp );

	if( !OUT_Init( XRES, YRES ) )
	{	
		#ifdef ERRORMSG
        MessageBox( 0, "OUT_Init()!",tlt_error,MB_OK|MB_ICONEXCLAMATION );
        #endif
		ExitProcess( 0 );
	}

    if( !RASTER_Do( doframe ) )
	{
        #ifdef ERRORMSG
        MessageBox( 0, "RASTER_Do()!",tlt_error,MB_OK|MB_ICONEXCLAMATION );
        #endif
		ExitProcess( 0 );
	}


//    intro_end();

    window_end( info );

    return( 0 );
}
